import time


def reporter(plan_data, testsuites_data, report_data, extra_data):

    extra_data['plan'] = plan_data['plan']
    extra_data['task'] = int(time.time() * 1000)

    testcases = []
    for key, ts in report_data.items():
        count = {'result': 'success', 'total': 0,
                 'success': 0, 'failure': 0, 'blocked': 0}
        no = 1
        for tc in ts:
            tc['testsuite'] = key
            tc['no'] = no
            no += 1
            tc = {**extra_data, **tc}

            res = tc['result'].lower()
            if tc['condition'].lower() in ('base', 'setup', 'snippet'):
                pass
            elif res in count:
                count[res] += 1
                count['total'] += 1
            testcases.append(tc)
            if count['failure'] + count['blocked']:
                count['result'] = 'failure'
        testsuites_data[key] = {**count, **testsuites_data[key]}

    testsuite = []
    count = {'total': 0, 'success': 0,
             'failure': 0, 'blocked': 0}
    result = 'success'
    for key, ts in testsuites_data.items():
        for k in count:
            count[k] += ts[k]
        if ts['result'] != 'success':
            result = 'failure'

        ts = {**extra_data, **ts}
        ts['testsuite'] = key
        testsuite.append(ts)
    count['result'] = result

    plan = {**extra_data, **count, **plan_data}

    return plan, testsuite, testcases


def local_time(timestamp):
    import time
    t = time.localtime(int(timestamp / 1000))
    return str(time.strftime("%Y/%m/%d %H:%M:%S", t))


def cost_time(start, end):
    return int((end - start) / 1000)


def summary(plan_data, testsuites_data, report_data, extra_data):
    plan, testsuites, testcases = reporter(
        plan_data, testsuites_data, report_data, extra_data)

    data = [['测试套件', '用例总数', '成功', '阻塞', '失败', '测试结果', '开始时间', '结束时间', '耗时(秒)']]
    failures = [['测试套件', '用例编号', '用例标题', '用例结果', '失败步骤', '备注']]

    for suite in testsuites:
        row = [suite['testsuite'], suite['total'], suite['success'], suite['blocked'], suite['failure'],
               suite['result'], local_time(suite['start_timestamp']), local_time(suite['end_timestamp']),
               cost_time(suite['start_timestamp'], suite['end_timestamp'])]
        data.append(row)

    flag = False
    for case in testcases:
        suite_name = '' if flag else case['testsuite']
        row = []
        if case['result'] == 'blocked':
            row = [suite_name, case['id'], case['title'], case['result']]
        elif case['result'] == 'failure':
            for step in case['steps']:
                if step['score'] == 'NO':
                    desc = '|'.join([step[k] for k in ('no', 'keyword', 'page', 'element')])
                    row = [suite_name, case['id'], case['title'], case['result'], desc, step['remark']]
                    break
        if row:
            flag = True
            failures.append(row)

    data.append(['--------'])
    total = ['总计', plan['total'], plan['success'], plan['blocked'], plan['failure'],
             plan['result'], local_time(plan['start_timestamp']), local_time(plan['end_timestamp']),
             cost_time(plan['start_timestamp'], plan['end_timestamp'])]

    data.append(total)
    if len(failures) > 1:
        data.append(['********'])
        data += failures
    return data



# coding=utf-8

# @Time: 2020/4/14 11:00
# @Auther: liyubin

import re
import os
import time
from super_sweetest.config import BASE_DIR

"""
使用xcodebuild,启动WebDriverAgent托管测试服务
devicename 不建议使用名称：iphone，否则将kill所有iphone名称服务
首次启动需xcode安装 WebDriverAgent app，并授权描述文件
"""


def logs_path():
    """项目路径 join build_logs"""
    path_ = os.path.join(BASE_DIR, 'build_logs')
    os.mkdir(path_) if not os.path.exists(path_) else path_
    return path_


def get_server_url(devicename):
    """
    获取ios_tagent_server输出信息中ServerURLHere
    :return: server_url
    :用法：connect_device("ios:///" + server_url)
    """

    print("----------- %s Get Server URL ----------------" % devicename)
    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    with open(build_log, 'rb')as fp:
        build_log = str(fp.read())

    serverurl = re.findall(r"http://(.+?)<-ServerURLHere", build_log)
    if serverurl:
        print("----------- %s ServerURLHere is %s ----------------" % (devicename, serverurl[0]))
        url = serverurl[0]
    else:
        print("----------- %s ServerURLHere is no ----------------" % devicename)
        url = 'retry server'
    return url


def run_ios_tagent_server(iostagen_file, platform, devicename):
    """
    非阻塞方式脱离终端在后台运行托管测试服务："nohup "+build_code
    通过xcodebuild 托管指定的手机的iOS-Tagent测试服务
    :return:
    # -derivedDataPath：产生的缓存文件放在./output目录下
    #  configuration：编译环境，选择Debug/Release
    # -destination :选择test时的目标设备和系统版本号
    """
    # build_code = "nohup xcodebuild test \
    #             -project %sWebDriverAgent.xcodeproj \
    #             -scheme WebDriverAgentRunner \
    #             -destination 'platform=%s,name=%s' &" % (iostagen_file, platform, devicename)

    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    # 实时写入日志
    build_code = "xcodebuild test \
                 -project %s/WebDriverAgent.xcodeproj \
                 -scheme WebDriverAgentRunner  \
                 -destination 'platform=%s,name=%s' > $'%s'" % (
        iostagen_file, platform, devicename, build_log)

    # print(build_code)
    print("----------- %s WebDriverAgent Server Start ----------------" % devicename)
    # 非阻塞方式脱离终端在后台运行托管测试服务
    os.popen("nohup %s &" % build_code)
    time.sleep(15)


def kill_ios_tagent_server(devicename):
    """
    杀掉 devicename 对应的WebDriverAgentRunner服务进程
    非阻塞
    :return:
    """
    print("----------- %s WebDriverAgent Server Stop ----------------" % devicename)
    kill_code = "ps -ef | grep '%s' | grep -v grep | awk '{print $2}' | xargs kill -9" % devicename
    os.popen("nohup %s &" % kill_code)

    build_log = os.path.join(logs_path(), '{}_build_log.txt'.format(devicename))
    if os.path.exists(build_log):
        os.remove(build_log)
    time.sleep(10)


if __name__ == '__main__':

    iostagen_file_ = '/Users/li/projects/ios_project/iOS-Tagent11'
    platform_ = 'iOS Simulator'  # 模拟器
    deviceName_ = 'iPhone 11'

    # platform_ = 'iOS'
    # deviceName_ = 'iPhone7'

    for i in range(1):
        #     print(i)
        run_ios_tagent_server(iostagen_file_, platform_, deviceName_)
        print(get_server_url(deviceName_))
        kill_ios_tagent_server(deviceName_)



import os
from time import sleep
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from super_sweetest.elements import e
from super_sweetest.globals import g
from super_sweetest.log import logger
from super_sweetest.config import element_wait_timeout


def locating_element(element, action=''):
    el_location = None
    try:
        el, value = e.get(element)
    except:
        logger.exception(
            'Locating the element:%s is Failure, this element is not define' % element)
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    if not isinstance(el, dict):
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    wait = WebDriverWait(g.driver, element_wait_timeout)

    if el['by'].lower() in ('title', 'url', 'current_url'):
        return None
    else:
        try:
            el_location = wait.until(EC.presence_of_element_located(
                (getattr(By, el['by'].upper()), value)))
        except:
            sleep(5)
            try:
                el_location = wait.until(EC.presence_of_element_located(
                    (getattr(By, el['by'].upper()), value)))
            except :
                raise Exception('Locating the element:%s is Failure: Timeout' % element)
    try:
        if g.driver.name in ('chrome', 'safari'):
            g.driver.execute_script(
                "arguments[0].scrollIntoViewIfNeeded(true)", el_location)
        else:
            g.driver.execute_script(
                "arguments[0].scrollIntoView(false)", el_location)
    except:
        pass

    try:
        if action == 'CLICK':
            #增加class_name elements 通过 index定位 点击
            if  isinstance(int(el['remark']),int):
                el_location = {'elements': el['by'], 'value': value, 'index_': int(el['remark'])}  # 反回所需参数到mobile.py
            else:
                el_location = wait.until(EC.element_to_be_clickable(
                    (getattr(By, el['by'].upper()), value)))
        else:
            # elements 输入 remark 是 int 执行index取值
            if isinstance(int(el['remark']), int):
            # if 'class_name' == el['by'] and isinstance(int(el['remark']), int):
                el_location = {'elements': el['by'],'value': value,'index_': int(el['remark'])}  #反回所需参数到mobile.py
            else:
                el_location = wait.until(EC.visibility_of_element_located(
                    (getattr(By, el['by'].upper()), value)))
    except:
        pass

    return el_location


def locating_elements(elements):
    elements_location = {}
    for el in elements:
        elements_location[el] = locating_element(el)
    return elements_location


def locating_data(keys):
    data_location = {}
    for key in keys:
        data_location[key] = locating_element(key)
    return data_location


def locating_air_element(step):
    element = step['element']
    try:
        el, value = e.get(element)
    except:
        logger.exception(
            'Locating the element:%s is Failure, this element is not define' % element)
        raise Exception(
            'Locating the element:%s is Failure, this element is not define' % element)

    if step['keyword'] in [ 'TAP', 'CHECK', 'NOTCHECK',  'WAIT_']:
        assert value.endswith('.png'), '当前图片：%s 格式不正确，正确 png 格式：photoname.png'% value
        el_location = os.path.join('./data',value)
    elif step['keyword'] in ['INPUT']:
        el_location = el['by'], value
    elif step['keyword'] == 'SWIPE':
        # 固定坐标滑动
        # assert re.match(r"^(?:[0-9]{1,4}\,){3}[0-9]{1,4}$", value), '当前坐标：%s 格式或数量不对，正确格式如：3000,20,3000,400'% value
        # 相对坐标滑动
        num_list = value.replace('，', ',').split(',')
        error_num_msg = '当前坐标：%s 格式或数量不对，正确格式如：0.2, 0.2, 2, 6\n' \
                                                                    'w,w,h,h\n' \
                                                                    'width  宽  w1 w2 小数  0.1 - 0.9\n' \
                                                                    'height 高  h1 h2 正数  1 - 9'% value
        assert (type(eval(num_list[0])) == float), error_num_msg
        assert (type(eval(num_list[1])) == float), error_num_msg
        assert (type(eval(num_list[2])) == int), error_num_msg
        assert (type(eval(num_list[3])) == int), error_num_msg

        el_location = num_list
    elif step['keyword'] == 'SWIPE_PHOTO':
        values = []
        value_ = value.replace('，',',').split(',')
        error_png_msg = '当前图片：%s 格式不正确，正确 图片滑动 格式：photoname.png#photoname1.png'% value
        if len(value_) != 2:
            raise error_png_msg
        for v in value_:
            assert v.endswith('.png'), error_png_msg
            values.append(os.path.join('./data',v))
        el_location = values
    elif step['keyword'] in  ['CLICK', 'CHECK_TEXT', 'NOTCHECK_TEXT']:
        el_location = el['by'], value
    elif step['keyword'] in ('BACK', 'HOME', 'MENU', 'POWER'):
        el_location = step['keyword']
    else:
        el_location = value

    return el_location


# coding=utf-8

# @Time: 2020/3/5 10:57
# @Auther: liyubin

from PIL import Image
import math
import operator
from functools import reduce
import base64


"""
处理图片信息
"""

def base64_photo(filename):
    """
    失败步骤图片转base64
    :param filename:
    :return:
    """
    with open(filename, 'rb')as fp:
        base64_data = 'data:image/png;base64,' + base64.b64encode(fp.read()).decode()
    return base64_data



def compare(pic1, pic2):
    """
    :param pic1: 图片1路径
    :param pic2: 图片2路径
    :return: 返回对比的结果
    """
    image1 = Image.open(pic1)
    image2 = Image.open(pic2)

    histogram1 = image1.histogram()
    histogram2 = image2.histogram()

    differ = math.sqrt(
        reduce(operator.add, list(map(lambda a, b: (a - b) ** 2, histogram1, histogram2))) / len(histogram1))

    print(differ)
    return differ


if __name__ == '__main__':

    differ = compare(r'./index_png.png', r'./index_png_1.png')
    if differ == 0.0:
        print('ok: ', differ)
    elif differ < 3.16:
        print('小于3.16: ', differ)
    else:
        print('fail: ', differ)



# coding: utf-8

# @Time:2020/3/15 12:34
# @Auther:sahala


from setuptools import setup, find_packages  # 这个包没有的可以pip一下

setup(
    name="super_sweetest",  # 这里是pip项目发布的名称
    version="1.1",  # 版本号，数值大的会优先被pip
    keywords=("pip", "super_sweetest", "featureextraction"),
    description="automation tools",
    long_description="automation tools: requests,appium,selenium,air_android,air_ios,adb,poco......",
    license="MIT Licence",

    url="https://github.com/",  # 项目相关文件地址，一般是github
    author="liyubin",
    author_email="1399393088@qq.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
                      "Appium-Python-Client",
                      "certifi",
                      "chardet",
                      "idna",
                      "injson",
                      "Pillow",
                      "requests",
                      "selenium",
                      "urllib3",
                      "xlrd",
                      "XlsxWriter",
                      "airtest",
                      "pocoui",
                      ]  # 这个项目需要的第三方库
)



# 步骤：

# 1.setup.py放在被打包同级
    # 本地打包项目文件
    # python setup.py sdist

# 2.上传项目到pypi服务器
    # pip install twine
    # twine upload dist/name.tar.gz

# 3.下载上传的库
    # pip install name



# coding=utf-8

# @Time: 2020/4/10 16:42
# @Auther: liyubin


from airtest.core.api import *
from poco.drivers.ios import iosPoco
from super_sweetest.servers.ios_tagent_server import kill_ios_tagent_server, run_ios_tagent_server
from super_sweetest.servers.ios_tagent_server import get_server_url


########################################
from airtest.core.settings import Settings as ST  # 全局伐值

ST.THRESHOLD_STRICT = 0.6  # assert_exists语句的默认阈值，一般比THRESHOLD更高一些
ST.THRESHOLD = 0.6  # 其他语句的伐值
ST.OPDELAY = 1  # 每一步操作后等待1秒再进行下一步操作
ST.FIND_TIMEOUT = 10  # 图像查找超时时间
ST.CVSTRATEGY = ["surf", "tpl", "brisk"]  # 修改图片识别算法顺序，只要成功匹配任意一个符合设定阙值的结果，程序就会认为识别成功

########################################


"""
airtest ios  服务
"""

class AirIos():


    def setup_air_ios(self, device_name, app_package, desired_caps):
        """
        connect_device airtest， init poco
        """

        auto_setup(__file__)

        # 获取配置
        IOSTAGEN_FILE = desired_caps.get('iostagen_file', '未配置')
        PLATFORM = desired_caps.get('platform', '未配置')

        # 服务启动
        self.setup_server(IOSTAGEN_FILE, PLATFORM, device_name)

        # 获取server_url
        server_url = get_server_url(device_name)
        if server_url == 'retry server':
            self.setup_server(IOSTAGEN_FILE, PLATFORM, device_name)
            server_url = get_server_url(device_name)

        # 通过ADB连接本地Android设备
        # init_device("Android")
        # 或者使用connect_device函数
        connect_device("ios:///" + server_url)

        # poco服务,先连接设备再初始化
        poco = iosPoco()

        start_app(app_package)

        sleep(8)
        return poco


    def setup_server(self, iostagen_file, platform, devicename):
        """airtest ios前置条件server"""
        kill_ios_tagent_server(devicename)
        run_ios_tagent_server(iostagen_file, platform, devicename)


    def teardown_air_ios(self, g):
        """关闭app，停止poco实例"""
        stop_app(g.appPackage)
        g.poco.stop_running()
        kill_ios_tagent_server(g.deviceName)


if __name__ == '__main__':
    desired_caps = {'iostagen_file_':'/Users/li/projects/ios_project/iOS-Tagent11d',
                    'platform':'iOS Simulator',
                    }
    iostagen_file_ = '/Users/li/projects/ios_project/iOS-Tagent11d'
    platform_ = 'iOS Simulator'  # 模拟器
    deviceName_ = 'iPhone 11'
    app_package = 'com.apple.Preferences'

    air = AirIos()
    air.setup_air_ios(iostagen_file_, platform_, desired_caps)
